@@ -24,8 +24,7 @@
 #include <string.h>
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gconf/gconf-client.h>
-#include <devkit-power-gobject/devicekit-power.h>
+#include <libupower-glib/upower.h>
 
 #include "egg-debug.h"
 
@@ -34,7 +33,6 @@
 #include "gpm-marshal.h"
 #include "gpm-engine.h"
 #include "gpm-stock-icons.h"
-#include "gpm-prefs-server.h"
 #include "gpm-phone.h"
 
 static void     gpm_engine_finalize   (GObject	  *object);
@@ -45,13 +43,13 @@ static void     gpm_engine_finalize   (GObject	  *object);
 
 struct GpmEnginePrivate
 {
-	GConfClient		*conf;
-	DkpClient		*client;
-	DkpDevice		*battery_composite;
+	GSettings		*settings;
+	UpClient		*client;
+	UpDevice		*battery_composite;
 	GPtrArray		*array;
 	GpmPhone		*phone;
 	GpmIconPolicy		 icon_policy;
-	gchar			*previous_icon;
+	GIcon			*previous_icon;
 	gchar			*previous_summary;
 
 	gboolean		 use_time_primary;
@@ -75,6 +73,7 @@ enum {
 	DISCHARGING,
 	LOW_CAPACITY,
 	PERHAPS_RECALL,
+	DEVICES_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -83,8 +82,8 @@ static gpointer gpm_engine_object = NULL;
 
 G_DEFINE_TYPE (GpmEngine, gpm_engine, G_TYPE_OBJECT)
 
-static DkpDevice *gpm_engine_get_composite_device (GpmEngine *engine, DkpDevice *original_device);
-static DkpDevice *gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_device);
+static UpDevice *gpm_engine_get_composite_device (GpmEngine *engine, UpDevice *original_device);
+static UpDevice *gpm_engine_update_composite_device (GpmEngine *engine, UpDevice *original_device);
 
 typedef enum {
 	GPM_ENGINE_WARNING_NONE = 0,
@@ -98,7 +97,7 @@ typedef enum {
  * gpm_engine_get_warning_csr:
  **/
 static GpmEngineWarning
-gpm_engine_get_warning_csr (GpmEngine *engine, DkpDevice *device)
+gpm_engine_get_warning_csr (GpmEngine *engine, UpDevice *device)
 {
 	gdouble percentage;
 
@@ -116,18 +115,13 @@ gpm_engine_get_warning_csr (GpmEngine *engine, DkpDevice *device)
  * gpm_engine_get_warning_percentage:
  **/
 static GpmEngineWarning
-gpm_engine_get_warning_percentage (GpmEngine *engine, DkpDevice *device)
+gpm_engine_get_warning_percentage (GpmEngine *engine, UpDevice *device)
 {
 	gdouble percentage;
 
 	/* get device properties */
 	g_object_get (device, "percentage", &percentage, NULL);
 
-	/* this is probably an error condition */
-	if (percentage == 0) {
-		egg_warning ("percentage zero, something's gone wrong");
-		return GPM_ENGINE_WARNING_NONE;
-	}
 	if (percentage <= engine->priv->action_percentage)
 		return GPM_ENGINE_WARNING_ACTION;
 	if (percentage <= engine->priv->critical_percentage)
@@ -141,20 +135,20 @@ gpm_engine_get_warning_percentage (GpmEngine *engine, DkpDevice *device)
  * gpm_engine_get_warning_time:
  **/
 static GpmEngineWarning
-gpm_engine_get_warning_time (GpmEngine *engine, DkpDevice *device)
+gpm_engine_get_warning_time (GpmEngine *engine, UpDevice *device)
 {
-	DkpDeviceType type;
+	UpDeviceKind kind;
 	gint64 time_to_empty;
 
 	/* get device properties */
 	g_object_get (device,
-		      "type", &type,
+		      "kind", &kind,
 		      "time-to-empty", &time_to_empty,
 		      NULL);
 
 	/* this is probably an error condition */
 	if (time_to_empty == 0) {
-		egg_debug ("time zero, falling back to percentage for %s", dkp_device_type_to_text (type));
+		egg_debug ("time zero, falling back to percentage for %s", up_device_kind_to_string (kind));
 		return gpm_engine_get_warning_percentage (engine, device);
 	}
 
@@ -176,15 +170,15 @@ gpm_engine_get_warning_time (GpmEngine *engine, DkpDevice *device)
  * Return value: A GpmEngine state, e.g. GPM_ENGINE_WARNING_DISCHARGING
  **/
 static GpmEngineWarning
-gpm_engine_get_warning (GpmEngine *engine, DkpDevice *device)
+gpm_engine_get_warning (GpmEngine *engine, UpDevice *device)
 {
-	DkpDeviceType type;
-	DkpDeviceState state;
+	UpDeviceKind kind;
+	UpDeviceState state;
 	GpmEngineWarning warning_type;
 
 	/* get device properties */
 	g_object_get (device,
-		      "type", &type,
+		      "kind", &kind,
 		      "state", &state,
 		      NULL);
 
@@ -192,24 +186,29 @@ gpm_engine_get_warning (GpmEngine *engine, DkpDevice *device)
 	warning_type = GPM_ENGINE_WARNING_NONE;
 
 	/* if the device in question is on ac, don't give a warning */
-	if (state == DKP_DEVICE_STATE_CHARGING)
+	if (state == UP_DEVICE_STATE_CHARGING)
 		goto out;
 
-	if (type == DKP_DEVICE_TYPE_MOUSE ||
-	    type == DKP_DEVICE_TYPE_KEYBOARD) {
+	if (kind == UP_DEVICE_KIND_MOUSE ||
+	    kind == UP_DEVICE_KIND_KEYBOARD) {
 
 		warning_type = gpm_engine_get_warning_csr (engine, device);
 
-	} else if (type == DKP_DEVICE_TYPE_UPS ||
-		   type == DKP_DEVICE_TYPE_PDA) {
+	} else if (kind == UP_DEVICE_KIND_UPS ||
+#if UP_CHECK_VERSION(0,9,5)
+		   kind == UP_DEVICE_KIND_MEDIA_PLAYER ||
+		   kind == UP_DEVICE_KIND_TABLET ||
+		   kind == UP_DEVICE_KIND_COMPUTER ||
+#endif
+		   kind == UP_DEVICE_KIND_PDA) {
 
 		warning_type = gpm_engine_get_warning_percentage (engine, device);
 
-	} else if (type == DKP_DEVICE_TYPE_PHONE) {
+	} else if (kind == UP_DEVICE_KIND_PHONE) {
 
 		warning_type = gpm_engine_get_warning_percentage (engine, device);
 
-	} else if (type == DKP_DEVICE_TYPE_BATTERY) {
+	} else if (kind == UP_DEVICE_KIND_BATTERY) {
 		/* only use the time when it is accurate, and GConf is not disabled */
 		if (engine->priv->use_time_primary)
 			warning_type = gpm_engine_get_warning_time (engine, device);
@@ -219,7 +218,7 @@ gpm_engine_get_warning (GpmEngine *engine, DkpDevice *device)
 
 	/* If we have no important engines, we should test for discharging */
 	if (warning_type == GPM_ENGINE_WARNING_NONE) {
-		if (state == DKP_DEVICE_STATE_DISCHARGING)
+		if (state == UP_DEVICE_STATE_DISCHARGING)
 			warning_type = GPM_ENGINE_WARNING_DISCHARGING;
 	}
 
@@ -239,9 +238,11 @@ gpm_engine_get_summary (GpmEngine *engine)
 {
 	guint i;
 	GPtrArray *array;
-	DkpDevice *device;
+	UpDevice *device;
+	UpDeviceState state;
 	GString *tooltip = NULL;
 	gchar *part;
+	gboolean is_present;
 
 	g_return_val_if_fail (GPM_IS_ENGINE (engine), NULL);
 
@@ -252,6 +253,14 @@ gpm_engine_get_summary (GpmEngine *engine)
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
+		g_object_get (device,
+			      "is-present", &is_present,
+			      "state", &state,
+			      NULL);
+		if (!is_present)
+			continue;
+		if (state == UP_DEVICE_STATE_EMPTY)
+			continue;
 		part = gpm_upower_get_device_summary (device);
 		if (part != NULL)
 			g_string_append_printf (tooltip, "%s\n", part);
@@ -271,15 +280,15 @@ gpm_engine_get_summary (GpmEngine *engine)
  *
  * Returns the icon
  **/
-static gchar *
-gpm_engine_get_icon_priv (GpmEngine *engine, DkpDeviceType device_type, GpmEngineWarning warning, gboolean use_state)
+static GIcon *
+gpm_engine_get_icon_priv (GpmEngine *engine, UpDeviceKind device_kind, GpmEngineWarning warning, gboolean use_state)
 {
 	guint i;
 	GPtrArray *array;
-	DkpDevice *device;
+	UpDevice *device;
 	GpmEngineWarning warning_temp;
-	DkpDeviceType type;
-	DkpDeviceState state;
+	UpDeviceKind kind;
+	UpDeviceState state;
 	gboolean is_present;
 
 	/* do we have specific device types? */
@@ -289,28 +298,28 @@ gpm_engine_get_icon_priv (GpmEngine *engine, DkpDeviceType device_type, GpmEngin
 
 		/* get device properties */
 		g_object_get (device,
-			      "type", &type,
+			      "kind", &kind,
 			      "state", &state,
 			      "is-present", &is_present,
 			      NULL);
 
 		/* if battery then use composite device to cope with multiple batteries */
-		if (type == DKP_DEVICE_TYPE_BATTERY)
+		if (kind == UP_DEVICE_KIND_BATTERY)
 			device = gpm_engine_get_composite_device (engine, device);
 
 		warning_temp = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-warning-old"));
-		if (type == device_type && is_present) {
+		if (kind == device_kind && is_present) {
 			if (warning != GPM_ENGINE_WARNING_NONE) {
 				if (warning_temp == warning)
-					return gpm_upower_get_device_icon (device);
+					return gpm_upower_get_device_icon (device, TRUE);
 				continue;
 			}
 			if (use_state) {
-				if (state == DKP_DEVICE_STATE_CHARGING || state == DKP_DEVICE_STATE_DISCHARGING)
-					return gpm_upower_get_device_icon (device);
+				if (state == UP_DEVICE_STATE_CHARGING || state == UP_DEVICE_STATE_DISCHARGING)
+					return gpm_upower_get_device_icon (device, TRUE);
 				continue;
 			}
-			return gpm_upower_get_device_icon (device);
+			return gpm_upower_get_device_icon (device, TRUE);
 		}
 	}
 	return NULL;
@@ -321,10 +330,10 @@ gpm_engine_get_icon_priv (GpmEngine *engine, DkpDeviceType device_type, GpmEngin
  *
  * Returns the icon
  **/
-gchar *
+GIcon *
 gpm_engine_get_icon (GpmEngine *engine)
 {
-	gchar *icon = NULL;
+	GIcon *icon = NULL;
 
 	g_return_val_if_fail (GPM_IS_ENGINE (engine), NULL);
 
@@ -335,16 +344,16 @@ gpm_engine_get_icon (GpmEngine *engine)
 	}
 
 	/* we try CRITICAL: BATTERY, UPS, MOUSE, KEYBOARD */
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_BATTERY, GPM_ENGINE_WARNING_CRITICAL, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_BATTERY, GPM_ENGINE_WARNING_CRITICAL, FALSE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_UPS, GPM_ENGINE_WARNING_CRITICAL, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_UPS, GPM_ENGINE_WARNING_CRITICAL, FALSE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_MOUSE, GPM_ENGINE_WARNING_CRITICAL, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_MOUSE, GPM_ENGINE_WARNING_CRITICAL, FALSE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_KEYBOARD, GPM_ENGINE_WARNING_CRITICAL, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_KEYBOARD, GPM_ENGINE_WARNING_CRITICAL, FALSE);
 	if (icon != NULL)
 		return icon;
 
@@ -355,16 +364,16 @@ gpm_engine_get_icon (GpmEngine *engine)
 	}
 
 	/* we try CRITICAL: BATTERY, UPS, MOUSE, KEYBOARD */
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_BATTERY, GPM_ENGINE_WARNING_LOW, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_BATTERY, GPM_ENGINE_WARNING_LOW, FALSE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_UPS, GPM_ENGINE_WARNING_LOW, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_UPS, GPM_ENGINE_WARNING_LOW, FALSE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_MOUSE, GPM_ENGINE_WARNING_LOW, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_MOUSE, GPM_ENGINE_WARNING_LOW, FALSE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_KEYBOARD, GPM_ENGINE_WARNING_LOW, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_KEYBOARD, GPM_ENGINE_WARNING_LOW, FALSE);
 	if (icon != NULL)
 		return icon;
 
@@ -375,10 +384,10 @@ gpm_engine_get_icon (GpmEngine *engine)
 	}
 
 	/* we try (DIS)CHARGING: BATTERY, UPS */
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_BATTERY, GPM_ENGINE_WARNING_NONE, TRUE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_BATTERY, GPM_ENGINE_WARNING_NONE, TRUE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_UPS, GPM_ENGINE_WARNING_NONE, TRUE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_UPS, GPM_ENGINE_WARNING_NONE, TRUE);
 	if (icon != NULL)
 		return icon;
 
@@ -389,10 +398,10 @@ gpm_engine_get_icon (GpmEngine *engine)
 	}
 
 	/* we try PRESENT: BATTERY, UPS */
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_BATTERY, GPM_ENGINE_WARNING_NONE, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_BATTERY, GPM_ENGINE_WARNING_NONE, FALSE);
 	if (icon != NULL)
 		return icon;
-	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_UPS, GPM_ENGINE_WARNING_NONE, FALSE);
+	icon = gpm_engine_get_icon_priv (engine, UP_DEVICE_KIND_UPS, GPM_ENGINE_WARNING_NONE, FALSE);
 	if (icon != NULL)
 		return icon;
 
@@ -402,9 +411,8 @@ gpm_engine_get_icon (GpmEngine *engine)
 		return NULL;
 	}
 
-	/* we fallback to the ac_adapter icon */
-	egg_debug ("Using fallback");
-	return g_strdup (GPM_STOCK_AC_ADAPTER);
+	/* do not show an icon */
+	return NULL;
 }
 
 /**
@@ -413,7 +421,7 @@ gpm_engine_get_icon (GpmEngine *engine)
 static gboolean
 gpm_engine_recalculate_state_icon (GpmEngine *engine)
 {
-	gchar *icon;
+	GIcon *icon;
 
 	g_return_val_if_fail (engine != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_ENGINE (engine), FALSE);
@@ -428,31 +436,29 @@ gpm_engine_recalculate_state_icon (GpmEngine *engine)
 		egg_debug ("** EMIT: icon-changed: none");
 		g_signal_emit (engine, signals [ICON_CHANGED], 0, NULL);
 
-		g_free (engine->priv->previous_icon);
+		g_object_unref (engine->priv->previous_icon);
 		engine->priv->previous_icon = NULL;
 		return TRUE;
 	}
 
 	/* no icon before, now icon */
 	if (engine->priv->previous_icon == NULL) {
-		egg_debug ("** EMIT: icon-changed: %s", icon);
 		g_signal_emit (engine, signals [ICON_CHANGED], 0, icon);
-		engine->priv->previous_icon = icon;
+		engine->priv->previous_icon = g_object_ref (icon);
 		return TRUE;
 	}
 
 	/* icon before, now different */
-	if (strcmp (engine->priv->previous_icon, icon) != 0) {
-		g_free (engine->priv->previous_icon);
-		engine->priv->previous_icon = icon;
-		egg_debug ("** EMIT: icon-changed: %s", icon);
+	if (!g_icon_equal (engine->priv->previous_icon, icon)) {
+		g_object_unref (engine->priv->previous_icon);
+		engine->priv->previous_icon = g_object_ref (icon);
 		g_signal_emit (engine, signals [ICON_CHANGED], 0, icon);
 		return TRUE;
 	}
 
 	egg_debug ("no change");
 	/* nothing to do */
-	g_free (icon);
+	g_object_unref (icon);
 	return FALSE;
 }
 
@@ -497,31 +503,25 @@ gpm_engine_recalculate_state (GpmEngine *engine)
 
 	gpm_engine_recalculate_state_icon (engine);
 	gpm_engine_recalculate_state_summary (engine);
+
+	g_signal_emit (engine, signals [DEVICES_CHANGED], 0);
 }
 
 /**
- * gpm_engine_conf_key_changed_cb:
+ * gpm_engine_settings_key_changed_cb:
  **/
 static void
-gpm_engine_conf_key_changed_cb (GConfClient *conf, guint cnxn_id, GConfEntry *entry, GpmEngine *engine)
+gpm_engine_settings_key_changed_cb (GSettings *settings, const gchar *key, GpmEngine *engine)
 {
-	GConfValue *value;
 	gchar *icon_policy;
 
-	if (entry == NULL)
-		return;
-	value = gconf_entry_get_value (entry);
-	if (value == NULL)
-		return;
+	if (g_strcmp0 (key, GPM_SETTINGS_USE_TIME_POLICY) == 0) {
+		engine->priv->use_time_primary = g_settings_get_boolean (settings, key);
 
-	if (strcmp (entry->key, GPM_CONF_USE_TIME_POLICY) == 0) {
-
-		engine->priv->use_time_primary = gconf_value_get_bool (value);
-
-	} else if (strcmp (entry->key, GPM_CONF_UI_ICON_POLICY) == 0) {
+	} else if (g_strcmp0 (key, GPM_SETTINGS_ICON_POLICY) == 0) {
 
 		/* do we want to display the icon in the tray */
-		icon_policy = gconf_client_get_string (conf, GPM_CONF_UI_ICON_POLICY, NULL);
+		icon_policy = g_settings_get_string (settings, key);
 		engine->priv->icon_policy = gpm_icon_policy_from_string (icon_policy);
 		g_free (icon_policy);
 
@@ -534,20 +534,20 @@ gpm_engine_conf_key_changed_cb (GConfClient *conf, guint cnxn_id, GConfEntry *en
  * gpm_engine_device_check_capacity:
  **/
 static gboolean
-gpm_engine_device_check_capacity (GpmEngine *engine, DkpDevice *device)
+gpm_engine_device_check_capacity (GpmEngine *engine, UpDevice *device)
 {
 	gboolean ret;
-	DkpDeviceType type;
+	UpDeviceKind kind;
 	gdouble capacity;
 
 	/* get device properties */
 	g_object_get (device,
-		      "type", &type,
+		      "kind", &kind,
 		      "capacity", &capacity,
 		      NULL);
 
 	/* not laptop battery */
-	if (type != DKP_DEVICE_TYPE_BATTERY)
+	if (kind != UP_DEVICE_KIND_BATTERY)
 		return FALSE;
 
 	/* capacity okay */
@@ -558,8 +558,8 @@ gpm_engine_device_check_capacity (GpmEngine *engine, DkpDevice *device)
 	if (capacity < 1.0f)
 		return FALSE;
 
-	/* only emit this if specified in gconf */
-	ret = gconf_client_get_bool (engine->priv->conf, GPM_CONF_NOTIFY_LOW_CAPACITY, NULL);
+	/* only emit this if specified in the settings */
+	ret = g_settings_get_boolean (engine->priv->settings, GPM_SETTINGS_NOTIFY_LOW_CAPACITY);
 	if (ret) {
 		egg_debug ("** EMIT: low-capacity");
 		g_signal_emit (engine, signals [LOW_CAPACITY], 0, device);
@@ -570,23 +570,29 @@ gpm_engine_device_check_capacity (GpmEngine *engine, DkpDevice *device)
 /**
  * gpm_engine_get_composite_device:
  **/
-static DkpDevice *
-gpm_engine_get_composite_device (GpmEngine *engine, DkpDevice *original_device)
+static UpDevice *
+gpm_engine_get_composite_device (GpmEngine *engine, UpDevice *original_device)
 {
 	guint battery_devices = 0;
 	GPtrArray *array;
-	DkpDevice *device;
-	DkpDeviceType type;
+	UpDevice *device;
+	UpDeviceKind kind;
+	UpDeviceKind original_kind;
 	guint i;
 
+	/* get the type of the original device */
+	g_object_get (original_device,
+		      "kind", &original_kind,
+		      NULL);
+
 	/* find out how many batteries in the system */
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
 		g_object_get (device,
-			      "type", &type,
+			      "kind", &kind,
 			      NULL);
-		if (type == DKP_DEVICE_TYPE_BATTERY)
+		if (kind == original_kind)
 			battery_devices++;
 	}
 
@@ -607,8 +613,8 @@ out:
 /**
  * gpm_engine_update_composite_device:
  **/
-static DkpDevice *
-gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_device)
+static UpDevice *
+gpm_engine_update_composite_device (GpmEngine *engine, UpDevice *original_device)
 {
 	guint i;
 	gdouble percentage;
@@ -625,39 +631,47 @@ gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_devic
 	gboolean is_discharging = FALSE;
 	gboolean is_fully_charged = TRUE;
 	GPtrArray *array;
-	DkpDevice *device;
-	DkpDeviceState state;
-	DkpDeviceType type;
+	UpDevice *device;
+	UpDeviceState state;
+	UpDeviceKind kind;
+	UpDeviceKind original_kind;
 	gboolean debug;
+	gchar *text;
 
 	/* are we printing to console? */
 	debug = egg_debug_enabled ();
 
+	/* get the type of the original device */
+	g_object_get (original_device,
+		      "kind", &original_kind,
+		      NULL);
+
 	/* update the composite device */
 	array = engine->priv->array;
 	for (i=0;i<array->len;i++) {
 		device = g_ptr_array_index (engine->priv->array, i);
 		g_object_get (device,
-			      "type", &type,
+			      "kind", &kind,
 			      "state", &state,
 			      "energy", &energy,
 			      "energy-full", &energy_full,
 			      "energy-rate", &energy_rate,
 			      NULL);
-		if (type != DKP_DEVICE_TYPE_BATTERY)
+		if (kind != original_kind)
 			continue;
 
 		if (debug) {
-			egg_debug ("printing device %i", i);
-			dkp_device_print (device);
+			text = up_device_to_text (device);
+			egg_debug ("printing device %i:\n%s", i, text);
+			g_free (text);
 		}
 
 		/* one of these will be charging or discharging */
-		if (state == DKP_DEVICE_STATE_CHARGING)
+		if (state == UP_DEVICE_STATE_CHARGING)
 			is_charging = TRUE;
-		if (state == DKP_DEVICE_STATE_DISCHARGING)
+		if (state == UP_DEVICE_STATE_DISCHARGING)
 			is_discharging = TRUE;
-		if (state != DKP_DEVICE_STATE_FULLY_CHARGED)
+		if (state != UP_DEVICE_STATE_FULLY_CHARGED)
 			is_fully_charged = FALSE;
 
 		/* sum up composite */
@@ -679,19 +693,19 @@ gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_devic
 
 	/* set composite state */
 	if (is_charging)
-		state = DKP_DEVICE_STATE_CHARGING;
+		state = UP_DEVICE_STATE_CHARGING;
 	else if (is_discharging)
-		state = DKP_DEVICE_STATE_DISCHARGING;
+		state = UP_DEVICE_STATE_DISCHARGING;
 	else if (is_fully_charged)
-		state = DKP_DEVICE_STATE_FULLY_CHARGED;
+		state = UP_DEVICE_STATE_FULLY_CHARGED;
 	else
-		state = DKP_DEVICE_STATE_UNKNOWN;
+		state = UP_DEVICE_STATE_UNKNOWN;
 
 	/* calculate a quick and dirty time remaining value */
 	if (energy_rate_total > 0) {
-		if (state == DKP_DEVICE_STATE_DISCHARGING)
+		if (state == UP_DEVICE_STATE_DISCHARGING)
 			time_to_empty = 3600 * (energy_total / energy_rate_total);
-		else if (state == DKP_DEVICE_STATE_CHARGING)
+		else if (state == UP_DEVICE_STATE_CHARGING)
 			time_to_full = 3600 * ((energy_full_total - energy_total) / energy_rate_total);
 	}
 
@@ -708,8 +722,11 @@ gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_devic
 		      "percentage", percentage,
 		      "state", state,
 		      NULL);
-	if (debug)
-		dkp_device_print (device);
+	if (debug) {
+		text = up_device_to_text (device);
+		egg_debug ("composite:\n%s", text);
+		g_free (text);
+	}
 
 	/* force update of icon */
 	gpm_engine_recalculate_state_icon (engine);
@@ -722,12 +739,12 @@ out:
  * gpm_engine_device_add:
  **/
 static void
-gpm_engine_device_add (GpmEngine *engine, DkpDevice *device)
+gpm_engine_device_add (GpmEngine *engine, UpDevice *device)
 {
 	GpmEngineWarning warning;
-	DkpDeviceState state;
-	DkpDeviceType type;
-	DkpDevice *composite;
+	UpDeviceState state;
+	UpDeviceKind kind;
+	UpDevice *composite;
 
 	/* assign warning */
 	warning = gpm_engine_get_warning (engine, device);
@@ -738,15 +755,15 @@ gpm_engine_device_add (GpmEngine *engine, DkpDevice *device)
 
 	/* get device properties */
 	g_object_get (device,
-		      "type", &type,
+		      "kind", &kind,
 		      "state", &state,
 		      NULL);
 
 	/* add old state for transitions */
-	egg_debug ("adding %s with state %s", dkp_device_get_object_path (device), dkp_device_state_to_text (state));
+	egg_debug ("adding %s with state %s", up_device_get_object_path (device), up_device_state_to_string (state));
 	g_object_set_data (G_OBJECT(device), "engine-state-old", GUINT_TO_POINTER(state));
 
-	if (type == DKP_DEVICE_TYPE_BATTERY) {
+	if (kind == UP_DEVICE_KIND_BATTERY) {
 		egg_debug ("updating because we added a device");
 		composite = gpm_engine_update_composite_device (engine, device);
 
@@ -762,25 +779,23 @@ gpm_engine_device_add (GpmEngine *engine, DkpDevice *device)
  * gpm_engine_check_recall:
  **/
 static gboolean
-gpm_engine_check_recall (GpmEngine *engine, DkpDevice *device)
+gpm_engine_check_recall (GpmEngine *engine, UpDevice *device)
 {
-	DkpDeviceType type;
+	UpDeviceKind kind;
 	gboolean recall_notice = FALSE;
 	gchar *recall_vendor = NULL;
 	gchar *recall_url = NULL;
 
 	/* get device properties */
 	g_object_get (device,
-		      "type", &type,
-#if DKP_CHECK_VERSION(0x009)
+		      "kind", &kind,
 		      "recall-notice", &recall_notice,
 		      "recall-vendor", &recall_vendor,
 		      "recall-url", &recall_url,
-#endif
 		      NULL);
 
 	/* not battery */
-	if (type != DKP_DEVICE_TYPE_BATTERY)
+	if (kind != UP_DEVICE_KIND_BATTERY)
 		goto out;
 
 	/* no recall data */
@@ -803,55 +818,37 @@ static gboolean
 gpm_engine_coldplug_idle_cb (GpmEngine *engine)
 {
 	guint i;
-	GPtrArray *array;
-	gboolean has_battery = FALSE;
-	gboolean has_ups = FALSE;
-	GpmPrefsServer *prefs_server;
-	DkpDevice *device;
-	DkpDeviceType type;
+	GPtrArray *array = NULL;
+	UpDevice *device;
+	gboolean ret;
+	GError *error = NULL;
 
 	g_return_val_if_fail (engine != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_ENGINE (engine), FALSE);
 
-	/* get array */
-	engine->priv->array = dkp_client_enumerate_devices (engine->priv->client, NULL);
-
-	/* do we have specific device types? */
-	array = engine->priv->array;
-	for (i=0;i<array->len;i++) {
-		device = g_ptr_array_index (engine->priv->array, i);
-
-		/* get device properties */
-		g_object_get (device,
-			      "type", &type,
-			      NULL);
-
-		if (type == DKP_DEVICE_TYPE_BATTERY)
-			has_battery = TRUE;
-		else if (type == DKP_DEVICE_TYPE_UPS)
-			has_ups = TRUE;
+	/* get devices from UPower */
+	ret = up_client_enumerate_devices_sync (engine->priv->client, NULL, &error);
+	if (!ret) {
+		egg_error ("failed to get device list: %s", error->message);
+		g_error_free (error);
+		goto out;
 	}
 
-	/* only show the battery prefs section if we have batteries */
-	prefs_server = gpm_prefs_server_new ();
-	if (has_battery)
-		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_BATTERY);
-	if (has_ups)
-		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_UPS);
-	g_object_unref (prefs_server);
-
 	/* connected mobile phones */
 	gpm_phone_coldplug (engine->priv->phone);
 
 	gpm_engine_recalculate_state (engine);
 
 	/* add to database */
+	array = up_client_get_devices (engine->priv->client);
 	for (i=0;i<array->len;i++) {
-		device = g_ptr_array_index (engine->priv->array, i);
+		device = g_ptr_array_index (array, i);
 		gpm_engine_device_add (engine, device);
 		gpm_engine_check_recall (engine, device);
 	}
-
+out:
+	if (array != NULL)
+		g_ptr_array_unref (array);
 	/* never repeat */
 	return FALSE;
 }
@@ -860,7 +857,7 @@ gpm_engine_coldplug_idle_cb (GpmEngine *engine)
  * gpm_engine_device_added_cb:
  **/
 static void
-gpm_engine_device_added_cb (DkpClient *client, DkpDevice *device, GpmEngine *engine)
+gpm_engine_device_added_cb (UpClient *client, UpDevice *device, GpmEngine *engine)
 {
 	/* add to list */
 	g_ptr_array_add (engine->priv->array, g_object_ref (device));
@@ -873,7 +870,7 @@ gpm_engine_device_added_cb (DkpClient *client, DkpDevice *device, GpmEngine *eng
  * gpm_engine_device_removed_cb:
  **/
 static void
-gpm_engine_device_removed_cb (DkpClient *client, DkpDevice *device, GpmEngine *engine)
+gpm_engine_device_removed_cb (UpClient *client, UpDevice *device, GpmEngine *engine)
 {
 	gboolean ret;
 	ret = g_ptr_array_remove (engine->priv->array, device);
@@ -887,22 +884,22 @@ gpm_engine_device_removed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
  * gpm_engine_device_changed_cb:
  **/
 static void
-gpm_engine_device_changed_cb (DkpClient *client, DkpDevice *device, GpmEngine *engine)
+gpm_engine_device_changed_cb (UpClient *client, UpDevice *device, GpmEngine *engine)
 {
-	DkpDeviceType type;
-	DkpDeviceState state;
-	DkpDeviceState state_old;
+	UpDeviceKind kind;
+	UpDeviceState state;
+	UpDeviceState state_old;
 	GpmEngineWarning warning_old;
 	GpmEngineWarning warning;
 
 	/* get device properties */
 	g_object_get (device,
-		      "type", &type,
+		      "kind", &kind,
 		      NULL);
 
 	/* if battery then use composite device to cope with multiple batteries */
-	if (type == DKP_DEVICE_TYPE_BATTERY) {
-		egg_debug ("updating because %s changed", dkp_device_get_object_path (device));
+	if (kind == UP_DEVICE_KIND_BATTERY) {
+		egg_debug ("updating because %s changed", up_device_get_object_path (device));
 		device = gpm_engine_update_composite_device (engine, device);
 	}
 
@@ -911,15 +908,15 @@ gpm_engine_device_changed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
 		      "state", &state,
 		      NULL);
 
-	egg_debug ("%s state is now %s", dkp_device_get_object_path (device), dkp_device_state_to_text (state));
+	egg_debug ("%s state is now %s", up_device_get_object_path (device), up_device_state_to_string (state));
 
 	/* see if any interesting state changes have happened */
 	state_old = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-state-old"));
 	if (state_old != state) {
-		if (state == DKP_DEVICE_STATE_DISCHARGING) {
+		if (state == UP_DEVICE_STATE_DISCHARGING) {
 			egg_debug ("** EMIT: discharging");
 			g_signal_emit (engine, signals [DISCHARGING], 0, device);
-		} else if (state == DKP_DEVICE_STATE_FULLY_CHARGED) {
+		} else if (state == UP_DEVICE_STATE_FULLY_CHARGED) {
 			egg_debug ("** EMIT: fully charged");
 			g_signal_emit (engine, signals [FULLY_CHARGED], 0, device);
 		}
@@ -952,7 +949,7 @@ gpm_engine_device_changed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
 /**
  * gpm_engine_get_devices:
  *
- * Return value: the DkpDevice array, free with g_ptr_array_unref()
+ * Return value: the UpDevice array, free with g_ptr_array_unref()
  **/
 GPtrArray *
 gpm_engine_get_devices (GpmEngine *engine)
@@ -960,20 +957,64 @@ gpm_engine_get_devices (GpmEngine *engine)
 	return g_ptr_array_ref (engine->priv->array);
 }
 
+/**
+ * gpm_engine_get_primary_device:
+ *
+ * Return value: the #UpDevice, free with g_object_unref()
+ **/
+UpDevice *
+gpm_engine_get_primary_device (GpmEngine *engine)
+{
+	guint i;
+	UpDevice *device = NULL;
+	UpDevice *device_tmp;
+	UpDeviceKind kind;
+	UpDeviceState state;
+	gboolean is_present;
+
+	for (i=0; i<engine->priv->array->len; i++) {
+		device_tmp = g_ptr_array_index (engine->priv->array, i);
+
+		/* get device properties */
+		g_object_get (device_tmp,
+			      "kind", &kind,
+			      "state", &state,
+			      "is-present", &is_present,
+			      NULL);
+
+		/* not present */
+		if (!is_present)
+			continue;
+
+		/* not discharging */
+		if (state != UP_DEVICE_STATE_DISCHARGING)
+			continue;
+
+		/* not battery */
+		if (kind != UP_DEVICE_KIND_BATTERY)
+			continue;
+
+		/* use composite device to cope with multiple batteries */
+		device = g_object_ref (gpm_engine_get_composite_device (engine, device_tmp));
+		break;
+	}
+	return device;
+}
+
 /**
  * phone_device_added_cb:
  **/
 static void
 phone_device_added_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 {
-	DkpDevice *device;
-	device = dkp_device_new ();
+	UpDevice *device;
+	device = up_device_new ();
 
 	egg_debug ("phone added %i", idx);
 
 	/* get device properties */
 	g_object_set (device,
-		      "type", DKP_DEVICE_TYPE_PHONE,
+		      "kind", UP_DEVICE_KIND_PHONE,
 		      "is-rechargeable", TRUE,
 		      "native-path", g_strdup_printf ("dummy:phone_%i", idx),
 		      "is-present", TRUE,
@@ -992,8 +1033,8 @@ static void
 phone_device_removed_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 {
 	guint i;
-	DkpDevice *device;
-	DkpDeviceType type;
+	UpDevice *device;
+	UpDeviceKind kind;
 
 	egg_debug ("phone removed %i", idx);
 
@@ -1002,10 +1043,10 @@ phone_device_removed_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 
 		/* get device properties */
 		g_object_get (device,
-			      "type", &type,
+			      "kind", &kind,
 			      NULL);
 
-		if (type == DKP_DEVICE_TYPE_PHONE) {
+		if (kind == UP_DEVICE_KIND_PHONE) {
 			g_ptr_array_remove_index (engine->priv->array, i);
 			break;
 		}
@@ -1022,9 +1063,9 @@ static void
 phone_device_refresh_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 {
 	guint i;
-	DkpDevice *device;
-	DkpDeviceType type;
-	DkpDeviceState state;
+	UpDevice *device;
+	UpDeviceKind kind;
+	UpDeviceState state;
 	gboolean is_present;
 	gdouble percentage;
 
@@ -1035,15 +1076,15 @@ phone_device_refresh_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 
 		/* get device properties */
 		g_object_get (device,
-			      "type", &type,
+			      "kind", &kind,
 			      "state", &state,
 			      "percentage", &percentage,
 			      "is-present", &is_present,
 			      NULL);
 
-		if (type == DKP_DEVICE_TYPE_PHONE) {
+		if (kind == UP_DEVICE_KIND_PHONE) {
 			is_present = gpm_phone_get_present (phone, idx);
-			state = gpm_phone_get_on_ac (phone, idx) ? DKP_DEVICE_STATE_CHARGING : DKP_DEVICE_STATE_DISCHARGING;
+			state = gpm_phone_get_on_ac (phone, idx) ? UP_DEVICE_STATE_CHARGING : UP_DEVICE_STATE_DISCHARGING;
 			percentage = gpm_phone_get_percentage (phone, idx);
 			break;
 		}
@@ -1061,11 +1102,12 @@ static void
 gpm_engine_init (GpmEngine *engine)
 {
 	gchar *icon_policy;
+	guint idle_id;
 
 	engine->priv = GPM_ENGINE_GET_PRIVATE (engine);
 
 	engine->priv->array = g_ptr_array_new_with_free_func (g_object_unref);
-	engine->priv->client = dkp_client_new ();
+	engine->priv->client = up_client_new ();
 	g_signal_connect (engine->priv->client, "device-added",
 			  G_CALLBACK (gpm_engine_device_added_cb), engine);
 	g_signal_connect (engine->priv->client, "device-removed",
@@ -1073,10 +1115,9 @@ gpm_engine_init (GpmEngine *engine)
 	g_signal_connect (engine->priv->client, "device-changed",
 			  G_CALLBACK (gpm_engine_device_changed_cb), engine);
 
-	engine->priv->conf = gconf_client_get_default ();
-	gconf_client_notify_add (engine->priv->conf, GPM_CONF_DIR,
-				 (GConfClientNotifyFunc) gpm_engine_conf_key_changed_cb,
-				 engine, NULL, NULL);
+	engine->priv->settings = g_settings_new (GPM_SETTINGS_SCHEMA);
+	g_signal_connect (engine->priv->settings, "changed",
+			  G_CALLBACK (gpm_engine_settings_key_changed_cb), engine);
 
 	engine->priv->phone = gpm_phone_new ();
 	g_signal_connect (engine->priv->phone, "device-added",
@@ -1087,9 +1128,9 @@ gpm_engine_init (GpmEngine *engine)
 			  G_CALLBACK (phone_device_refresh_cb), engine);
 
 	/* create a fake virtual composite battery */
-	engine->priv->battery_composite = dkp_device_new ();
+	engine->priv->battery_composite = up_device_new ();
 	g_object_set (engine->priv->battery_composite,
-		      "type", DKP_DEVICE_TYPE_BATTERY,
+		      "kind", UP_DEVICE_KIND_BATTERY,
 		      "is-rechargeable", TRUE,
 		      "native-path", "dummy:composite_battery",
 		      "power-supply", TRUE,
@@ -1100,28 +1141,31 @@ gpm_engine_init (GpmEngine *engine)
 	engine->priv->previous_summary = NULL;
 
 	/* do we want to display the icon in the tray */
-	icon_policy = gconf_client_get_string (engine->priv->conf, GPM_CONF_UI_ICON_POLICY, NULL);
+	icon_policy = g_settings_get_string (engine->priv->settings, GPM_SETTINGS_ICON_POLICY);
 	engine->priv->icon_policy = gpm_icon_policy_from_string (icon_policy);
 	g_free (icon_policy);
 
 	/* get percentage policy */
-	engine->priv->low_percentage = gconf_client_get_int (engine->priv->conf, GPM_CONF_THRESH_PERCENTAGE_LOW, NULL);
-	engine->priv->critical_percentage = gconf_client_get_int (engine->priv->conf, GPM_CONF_THRESH_PERCENTAGE_CRITICAL, NULL);
-	engine->priv->action_percentage = gconf_client_get_int (engine->priv->conf, GPM_CONF_THRESH_PERCENTAGE_ACTION, NULL);
+	engine->priv->low_percentage = g_settings_get_int (engine->priv->settings, GPM_SETTINGS_PERCENTAGE_LOW);
+	engine->priv->critical_percentage = g_settings_get_int (engine->priv->settings, GPM_SETTINGS_PERCENTAGE_CRITICAL);
+	engine->priv->action_percentage = g_settings_get_int (engine->priv->settings, GPM_SETTINGS_PERCENTAGE_ACTION);
 
 	/* get time policy */
-	engine->priv->low_time = gconf_client_get_int (engine->priv->conf, GPM_CONF_THRESH_TIME_LOW, NULL);
-	engine->priv->critical_time = gconf_client_get_int (engine->priv->conf, GPM_CONF_THRESH_TIME_CRITICAL, NULL);
-	engine->priv->action_time = gconf_client_get_int (engine->priv->conf, GPM_CONF_THRESH_TIME_ACTION, NULL);
+	engine->priv->low_time = g_settings_get_int (engine->priv->settings, GPM_SETTINGS_TIME_LOW);
+	engine->priv->critical_time = g_settings_get_int (engine->priv->settings, GPM_SETTINGS_TIME_CRITICAL);
+	engine->priv->action_time = g_settings_get_int (engine->priv->settings, GPM_SETTINGS_TIME_ACTION);
 
 	/* we can disable this if the time remaining is inaccurate or just plain wrong */
-	engine->priv->use_time_primary = gconf_client_get_bool (engine->priv->conf, GPM_CONF_USE_TIME_POLICY, NULL);
+	engine->priv->use_time_primary = g_settings_get_boolean (engine->priv->settings, GPM_SETTINGS_USE_TIME_POLICY);
 	if (engine->priv->use_time_primary)
 		egg_debug ("Using per-time notification policy");
 	else
 		egg_debug ("Using percentage notification policy");
 
-	g_idle_add ((GSourceFunc) gpm_engine_coldplug_idle_cb, engine);
+	idle_id = g_idle_add ((GSourceFunc) gpm_engine_coldplug_idle_cb, engine);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (idle_id, "[GpmEngine] coldplug");
+#endif
 }
 
 /**
@@ -1140,8 +1184,8 @@ gpm_engine_class_init (GpmEngineClass *klass)
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmEngineClass, icon_changed),
-			      NULL, NULL, g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1, G_TYPE_STRING);
+			      NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, G_TYPE_ICON);
 	signals [SUMMARY_CHANGED] =
 		g_signal_new ("summary-changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -1199,6 +1243,13 @@ gpm_engine_class_init (GpmEngineClass *klass)
 			      G_STRUCT_OFFSET (GpmEngineClass, charge_critical),
 			      NULL, NULL, g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+	signals [DEVICES_CHANGED] =
+		g_signal_new ("devices-changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GpmEngineClass, devices_changed),
+			      NULL, NULL, g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 }
 
 /**
@@ -1221,7 +1272,8 @@ gpm_engine_finalize (GObject *object)
 	g_object_unref (engine->priv->phone);
 	g_object_unref (engine->priv->battery_composite);
 
-	g_free (engine->priv->previous_icon);
+	if (engine->priv->previous_icon != NULL)
+		g_object_unref (engine->priv->previous_icon);
 	g_free (engine->priv->previous_summary);
 
 	G_OBJECT_CLASS (gpm_engine_parent_class)->finalize (object);
