@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -30,34 +30,35 @@
 #include "gpm-screensaver.h"
 #include "gpm-common.h"
 #include "egg-debug.h"
-#include "egg-dbus-proxy.h"
 
 static void     gpm_screensaver_finalize   (GObject		*object);
 
 #define GPM_SCREENSAVER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_SCREENSAVER, GpmScreensaverPrivate))
 
 #define GS_LISTENER_SERVICE	"org.gnome.ScreenSaver"
-#define GS_LISTENER_PATH	"/org/gnome/ScreenSaver"
+#define GS_LISTENER_PATH	"/"
 #define GS_LISTENER_INTERFACE	"org.gnome.ScreenSaver"
 
 struct GpmScreensaverPrivate
 {
-	EggDbusProxy		*gproxy;
+	DBusGProxy		*proxy;
 	GConfClient		*conf;
 };
 
 enum {
-	GS_DELAY_CHANGED,
-	CONNECTION_CHANGED,
 	AUTH_REQUEST,
 	LAST_SIGNAL
 };
 
+#if 0
 static guint signals [LAST_SIGNAL] = { 0 };
+#endif
 static gpointer gpm_screensaver_object = NULL;
 
 G_DEFINE_TYPE (GpmScreensaver, gpm_screensaver, G_TYPE_OBJECT)
 
+#if 0
+
 /** Invoked when we get the AuthenticationRequestBegin from g-s when the user
  *  has moved their mouse and we are showing the authentication box.
  */
@@ -87,28 +88,25 @@ gpm_screensaver_auth_end (DBusGProxy     *proxy,
 static gboolean
 gpm_screensaver_proxy_connect_more (GpmScreensaver *screensaver)
 {
-	DBusGProxy *proxy;
-
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
-	if (proxy == NULL) {
+	if (screensaver->priv->proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
 	}
 
-	g_signal_emit (screensaver, signals [CONNECTION_CHANGED], 0, TRUE);
-
 	/* get AuthenticationRequestBegin */
-	dbus_g_proxy_add_signal (proxy, "AuthenticationRequestBegin", G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (proxy,
+	dbus_g_proxy_add_signal (screensaver->priv->proxy,
+				 "AuthenticationRequestBegin", G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (screensaver->priv->proxy,
 				     "AuthenticationRequestBegin",
 				     G_CALLBACK (gpm_screensaver_auth_begin),
 				     screensaver, NULL);
 
 	/* get AuthenticationRequestEnd */
-	dbus_g_proxy_add_signal (proxy, "AuthenticationRequestEnd", G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (proxy,
+	dbus_g_proxy_add_signal (screensaver->priv->proxy,
+				 "AuthenticationRequestEnd", G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (screensaver->priv->proxy,
 				     "AuthenticationRequestEnd",
 				     G_CALLBACK (gpm_screensaver_auth_end),
 				     screensaver, NULL);
@@ -124,11 +122,10 @@ static gboolean
 gpm_screensaver_proxy_disconnect_more (GpmScreensaver *screensaver)
 {
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
-
-	g_signal_emit (screensaver, signals [CONNECTION_CHANGED], 0, FALSE);
 	egg_debug ("gnome-screensaver disconnected from the session DBUS");
 	return TRUE;
 }
+#endif
 
 /**
  * gpm_screensaver_lock_enabled:
@@ -140,25 +137,10 @@ gpm_screensaver_lock_enabled (GpmScreensaver *screensaver)
 {
 	gboolean enabled;
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
-
-	enabled = gconf_client_get_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, NULL);
-
+	enabled = gconf_client_get_bool (screensaver->priv->conf, GS_CONF_PREF_LOCK_ENABLED, NULL);
 	return enabled;
 }
 
-/**
- * gpm_screensaver_lock_set:
- * @screensaver: This class instance
- * @lock: If gnome-screensaver should lock the screen on screensave
- **/
-gboolean
-gpm_screensaver_lock_set (GpmScreensaver *screensaver, gboolean lock)
-{
-	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
-	gconf_client_set_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, lock, NULL);
-	return TRUE;
-}
-
 /**
  * gpm_screensaver_lock
  * @screensaver: This class instance
@@ -168,18 +150,17 @@ gboolean
 gpm_screensaver_lock (GpmScreensaver *screensaver)
 {
 	guint sleepcount = 0;
-	DBusGProxy *proxy;
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
-	if (proxy == NULL) {
+	if (screensaver->priv->proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
 	}
 
 	egg_debug ("doing gnome-screensaver lock");
-	dbus_g_proxy_call_no_reply (proxy, "Lock", G_TYPE_INVALID);
+	dbus_g_proxy_call_no_reply (screensaver->priv->proxy,
+				    "Lock", G_TYPE_INVALID);
 
 	/* When we send the Lock signal to g-ss it takes maybe a second
 	   or so to fade the screen and lock. If we suspend mid fade then on
@@ -193,6 +174,7 @@ gpm_screensaver_lock (GpmScreensaver *screensaver)
 		/* Sleep for 1/10s */
 		g_usleep (1000 * 100);
 		if (sleepcount++ > 50) {
+			egg_debug ("timeout waiting for gnome-screensaver");
 			break;
 		}
 	}
@@ -213,24 +195,17 @@ gpm_screensaver_add_throttle (GpmScreensaver *screensaver,
 	GError  *error = NULL;
 	gboolean ret;
 	guint32  cookie;
-	DBusGProxy *proxy;
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), 0);
 	g_return_val_if_fail (reason != NULL, 0);
 
-	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
-	if (proxy == NULL) {
+	if (screensaver->priv->proxy == NULL) {
 		egg_warning ("not connected");
 		return 0;
 	}
 
-	/* shouldn't be, but make sure proxy valid */
-	if (proxy == NULL) {
-		egg_warning ("g-s proxy is NULL!");
-		return 0;
-	}
-
-	ret = dbus_g_proxy_call (proxy, "Throttle", &error,
+	ret = dbus_g_proxy_call (screensaver->priv->proxy,
+				 "Throttle", &error,
 				 G_TYPE_STRING, "Power screensaver",
 				 G_TYPE_STRING, reason,
 				 G_TYPE_INVALID,
@@ -250,24 +225,25 @@ gpm_screensaver_add_throttle (GpmScreensaver *screensaver,
 	return cookie;
 }
 
+/**
+ * gpm_screensaver_remove_throttle:
+ **/
 gboolean
-gpm_screensaver_remove_throttle (GpmScreensaver *screensaver,
-				 guint           cookie)
+gpm_screensaver_remove_throttle (GpmScreensaver *screensaver, guint cookie)
 {
 	gboolean ret;
-	DBusGProxy *proxy;
 	GError *error = NULL;
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
-	if (proxy == NULL) {
+	if (screensaver->priv->proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
 	}
 
 	egg_debug ("removing throttle: id %u", cookie);
-	ret = dbus_g_proxy_call (proxy, "UnThrottle", &error,
+	ret = dbus_g_proxy_call (screensaver->priv->proxy,
+				 "UnThrottle", &error,
 				 G_TYPE_UINT, cookie,
 				 G_TYPE_INVALID,
 				 G_TYPE_INVALID);
@@ -294,18 +270,17 @@ gpm_screensaver_check_running (GpmScreensaver *screensaver)
 {
 	gboolean ret;
 	gboolean temp = TRUE;
-	DBusGProxy *proxy;
 	GError *error = NULL;
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
-	if (proxy == NULL) {
+	if (screensaver->priv->proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
 	}
 
-	ret = dbus_g_proxy_call (proxy, "GetActive", &error,
+	ret = dbus_g_proxy_call (screensaver->priv->proxy,
+				 "GetActive", &error,
 				 G_TYPE_INVALID,
 				 G_TYPE_BOOLEAN, &temp,
 				 G_TYPE_INVALID);
@@ -328,18 +303,15 @@ gpm_screensaver_check_running (GpmScreensaver *screensaver)
 gboolean
 gpm_screensaver_poke (GpmScreensaver *screensaver)
 {
-	DBusGProxy *proxy;
-
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
-	if (proxy == NULL) {
+	if (screensaver->priv->proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
 	}
 
 	egg_debug ("poke");
-	dbus_g_proxy_call_no_reply (proxy,
+	dbus_g_proxy_call_no_reply (screensaver->priv->proxy,
 				    "SimulateUserActivity",
 				    G_TYPE_INVALID);
 	return TRUE;
@@ -356,26 +328,7 @@ gpm_screensaver_class_init (GpmScreensaverClass *klass)
 	object_class->finalize = gpm_screensaver_finalize;
 	g_type_class_add_private (klass, sizeof (GpmScreensaverPrivate));
 
-	signals [GS_DELAY_CHANGED] =
-		g_signal_new ("gs-delay-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmScreensaverClass, gs_delay_changed),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__INT,
-			      G_TYPE_NONE, 1, G_TYPE_INT);
-
-	signals [CONNECTION_CHANGED] =
-		g_signal_new ("connection-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmScreensaverClass, connection_changed),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
-
+#if 0
 	signals [AUTH_REQUEST] =
 		g_signal_new ("auth-request",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -385,26 +338,27 @@ gpm_screensaver_class_init (GpmScreensaverClass *klass)
 			      NULL,
 			      g_cclosure_marshal_VOID__BOOLEAN,
 			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+#endif
 }
 
 /**
- * proxy_status_cb:
+ * gpm_screensaver_proxy_status_cb:
  * @proxy: The dbus raw proxy
  * @status: The status of the service, where TRUE is connected
  * @screensaver: This class instance
  **/
 static void
-proxy_status_cb (DBusGProxy     *proxy,
-		 gboolean	 status,
-		 GpmScreensaver *screensaver)
+gpm_screensaver_proxy_status_cb (DBusGProxy *proxy, gboolean status, GpmScreensaver *screensaver)
 {
 	g_return_if_fail (GPM_IS_SCREENSAVER (screensaver));
 
+#if 0
 	if (status) {
 		gpm_screensaver_proxy_connect_more (screensaver);
 	} else {
 		gpm_screensaver_proxy_disconnect_more (screensaver);
 	}
+#endif
 }
 
 /**
@@ -414,22 +368,21 @@ proxy_status_cb (DBusGProxy     *proxy,
 static void
 gpm_screensaver_init (GpmScreensaver *screensaver)
 {
-//	DBusGProxy *proxy;
-//	DBusGConnection *connection;
+	DBusGConnection *connection;
 
 	screensaver->priv = GPM_SCREENSAVER_GET_PRIVATE (screensaver);
 
-	screensaver->priv->gproxy = egg_dbus_proxy_new ();
-//	connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
-//	proxy = egg_dbus_proxy_assign (screensaver->priv->gproxy, connection, GS_LISTENER_SERVICE,
-//				       GS_LISTENER_PATH, GS_LISTENER_INTERFACE);
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
 
-	g_signal_connect (screensaver->priv->gproxy, "proxy-status",
-			  G_CALLBACK (proxy_status_cb),
+	screensaver->priv->proxy = dbus_g_proxy_new_for_name (connection,
+							      GS_LISTENER_SERVICE,
+							      GS_LISTENER_PATH,
+							      GS_LISTENER_INTERFACE);
+
+	g_signal_connect (screensaver->priv->proxy, "proxy-status",
+			  G_CALLBACK (gpm_screensaver_proxy_status_cb),
 			  screensaver);
 
-	gpm_screensaver_proxy_connect_more (screensaver);
-
 	screensaver->priv->conf = gconf_client_get_default ();
 }
 
@@ -447,9 +400,11 @@ gpm_screensaver_finalize (GObject *object)
 	screensaver = GPM_SCREENSAVER (object);
 	screensaver->priv = GPM_SCREENSAVER_GET_PRIVATE (screensaver);
 
+#if 0
 	gpm_screensaver_proxy_disconnect_more (screensaver);
+#endif
 	g_object_unref (screensaver->priv->conf);
-	g_object_unref (screensaver->priv->gproxy);
+	g_object_unref (screensaver->priv->proxy);
 
 	G_OBJECT_CLASS (gpm_screensaver_parent_class)->finalize (object);
 }
@@ -469,3 +424,59 @@ gpm_screensaver_new (void)
 	}
 	return GPM_SCREENSAVER (gpm_screensaver_object);
 }
+/***************************************************************************
+ ***                          MAKE CHECK TESTS                           ***
+ ***************************************************************************/
+#ifdef EGG_TEST
+#include "egg-test.h"
+
+#if 0
+static gboolean test_got_request = FALSE;
+static void
+gpm_screensaver_test_auth_request_cb (GpmScreensaver *screensaver, gboolean auth, EggTest *test)
+{
+	egg_debug ("auth request = %i", auth);
+	test_got_request = auth;
+
+	egg_test_loop_quit (test);
+}
+#endif
+
+void
+gpm_screensaver_test (gpointer data)
+{
+	GpmScreensaver *screensaver;
+//	guint value;
+	gboolean ret;
+	EggTest *test = (EggTest *) data;
+
+	if (egg_test_start (test, "GpmScreensaver") == FALSE)
+		return;
+
+	/************************************************************/
+	egg_test_title (test, "make sure we get a non null screensaver");
+	screensaver = gpm_screensaver_new ();
+	egg_test_assert (test, (screensaver != NULL));
+
+#if 0
+	/* connect signals */
+	g_signal_connect (screensaver, "auth-request",
+			  G_CALLBACK (gpm_screensaver_test_auth_request_cb), test);
+#endif
+	/************************************************************/
+	egg_test_title (test, "lock screensaver");
+	ret = gpm_screensaver_lock (screensaver);
+	egg_test_assert (test, ret);
+
+	/************************************************************/
+	egg_test_title (test, "poke screensaver");
+	ret = gpm_screensaver_poke (screensaver);
+	egg_test_assert (test, ret);
+
+	g_object_unref (screensaver);
+
+	egg_test_end (test);
+}
+
+#endif
+
